@@ -1,16 +1,17 @@
 import logging
-from datetime import datetime, timedelta
+from datetime import datetime, timedelta, timezone
 from typing import Optional, Tuple
 
 import dateparser
 import pytz
+from apscheduler.job import Job
 from nio import AsyncClient, MatrixRoom
 from nio.events.room_events import RoomMessageText
 from readabledelta import readabledelta
 
-from matrix_reminder_bot.chat_functions import send_text_to_room
-from matrix_reminder_bot.config import Config
-from matrix_reminder_bot.errors import CommandError
+from matrix_reminder_bot.config import CONFIG
+from matrix_reminder_bot.errors import CommandError, CommandSyntaxError
+from matrix_reminder_bot.functions import command_syntax, send_text_to_room
 from matrix_reminder_bot.reminder import ALARMS, REMINDERS, SCHEDULER, Reminder
 from matrix_reminder_bot.storage import Storage
 
@@ -22,7 +23,6 @@ class Command(object):
         self,
         client: AsyncClient,
         store: Storage,
-        config: Config,
         command: str,
         room: MatrixRoom,
         event: RoomMessageText,
@@ -32,19 +32,17 @@ class Command(object):
         Args:
             client: The client to communicate to matrix with
             store: Bot storage
-            config: Bot configuration parameters
             command: The command and arguments
             room: The room the command was sent in
             event: The event describing the command
         """
         self.client = client
         self.store = store
-        self.config = config
         self.room = room
         self.event = event
 
         msg_without_prefix = command[
-            len(config.command_prefix) :
+            len(CONFIG.command_prefix) :
         ]  # Remove the cmd prefix
         self.args = (
             msg_without_prefix.split()
@@ -88,7 +86,10 @@ class Command(object):
         args_str = " ".join(self.args)
         logger.debug("Parsing command arguments: %s", args_str)
 
-        time_str, reminder_text = args_str.split(";", maxsplit=1)
+        try:
+            time_str, reminder_text = args_str.split(";", maxsplit=1)
+        except ValueError:
+            raise CommandSyntaxError()
         logger.debug("Got time: %s", time_str)
 
         # Clean up the input
@@ -110,8 +111,8 @@ class Command(object):
 
             # Generate a timedelta between now and the recurring time
             # `recurse_time` is guaranteed to always be in the future
-            # Round datetime.now() to the nearest second for better time display
-            current_time = datetime.now().replace(microsecond=0)
+            current_time = self._get_datetime_now(CONFIG.timezone)
+
             recurse_timedelta = recurse_time - current_time
             logger.debug("Recurring timedelta: %s", recurse_timedelta)
 
@@ -140,13 +141,17 @@ class Command(object):
         """
         time = dateparser.parse(
             time_str,
-            settings={"PREFER_DATES_FROM": "future", "TIMEZONE": self.config.timezone},
+            settings={
+                "PREFER_DATES_FROM": "future",
+                "TIMEZONE": CONFIG.timezone,
+                "RETURN_AS_TIMEZONE_AWARE": True,
+            },
         )
         if not time:
             raise CommandError(f"The given time '{time_str}' is invalid.")
 
         # Disallow times in the past
-        if time < datetime.now():
+        if time < self._get_datetime_now(CONFIG.timezone):
             raise CommandError(f"The given time '{time_str}' is in the past.")
 
         # Round datetime object to the nearest second for nicer display
@@ -154,6 +159,20 @@ class Command(object):
 
         return time
 
+    def _get_datetime_now(self, tz: str) -> datetime:
+        """Returns a timezone-aware datetime object of the current time"""
+        # Get a datetime with no timezone information
+        no_timezone_datetime = datetime(2009, 9, 1)
+
+        # Create a datetime.timezone object with the correct offset from UTC
+        offset = timezone(pytz.timezone(tz).utcoffset(no_timezone_datetime))
+
+        # Get datetime.now with that offset
+        now = datetime.now(offset)
+
+        # Round to the nearest second for nicer display
+        return now.replace(microsecond=0)
+
     async def _confirm_reminder(self, reminder: Reminder):
         """Sends a message to the room confirming the reminder is set
 
@@ -183,17 +202,19 @@ class Command(object):
 
         if reminder.recurse_timedelta:
             # Inform the user how often their reminder will repeat
-            human_readable_recurse_timedelta = self._get_readable_timedelta(
-                reminder.start_time,
-                pytz.timezone(reminder.timezone),
-                reminder.recurse_timedelta,
-            )
-
-            text += f", and again every {human_readable_recurse_timedelta}"
+            text += f", and again every {readabledelta(reminder.recurse_timedelta)}"
 
         # Add some punctuation
         text += "!"
 
+        if reminder.alarm:
+            # Inform the user that an alarm is attached to this reminder
+            text += (
+                f"\n\nWhen this reminder goes off, an alarm will sound every "
+                f"5 minutes until silenced. Alarms may be silenced using the "
+                f"`{CONFIG.command_prefix}silence` command."
+            )
+
         # Send the message to the room
         await send_text_to_room(self.client, self.room.room_id, text)
 
@@ -250,7 +271,7 @@ class Command(object):
             self.room.room_id,
             reminder_text,
             start_time=start_time,
-            timezone=self.config.timezone,
+            timezone=CONFIG.timezone,
             cron_tab=cron_tab,
             recurse_timedelta=recurse_timedelta,
             target_user=target,
@@ -264,16 +285,6 @@ class Command(object):
         # Send a message to the room confirming the creation of the reminder
         await self._confirm_reminder(reminder)
 
-    def _get_readable_timedelta(
-        self, dt: datetime, timezone: datetime.tzinfo, delta: timedelta,
-    ) -> str:
-        """Returns a human-readable timedelta string, accounting for timezones"""
-        # Offset the timedelta by the given timezone
-        delta = delta - timezone.utcoffset(dt)
-
-        # Get the human-readable version
-        return readabledelta(delta)
-
     async def process(self):
         """Process the command"""
         if self.command == "remindme":
@@ -302,54 +313,88 @@ class Command(object):
         elif self.command == "help":
             await self._help()
 
+    @command_syntax("[every <recurring time>;] <start time>; <reminder text>")
     async def _remind_me(self):
         """Set a reminder that will remind only the user who created it"""
         await self._remind(target=self.event.sender)
 
+    @command_syntax("[every <recurring time>;] <start time>; <reminder text>")
     async def _remind_room(self):
         """Set a reminder that will mention the room that the reminder was created in"""
         await self._remind()
 
+    @command_syntax("[every <recurring time>;] <start time>; <reminder text>")
     async def _alarm_me(self):
         """Set a reminder with an alarm that will remind only the user who created it"""
         await self._remind(target=self.event.sender, alarm=True)
 
+    @command_syntax("[every <recurring time>;] <start time>; <reminder text>")
     async def _alarm_room(self):
         """Set a reminder with an alarm that when fired will mention the room that the
         reminder was created in
         """
         await self._remind(alarm=True)
 
+    @command_syntax("[<reminder text>]")
     async def _silence(self):
         """Silences an ongoing alarm"""
+        alarm_job = None
+
         # Attempt to find a reminder with an alarm currently going off
         reminder_text = " ".join(self.args)
-        alarm_job = ALARMS.get((self.room.room_id, reminder_text.upper()))
+        if reminder_text:
+            # Find the alarm job via its reminder text
+            alarm_job = ALARMS.get((self.room.room_id, reminder_text.upper()))
 
-        if alarm_job:
-            # We found a reminder with an alarm
-            ALARMS.pop((self.room.room_id, reminder_text.upper()), None)
-
-            if SCHEDULER.get_job(alarm_job.id):
-                # Silence the alarm
-                alarm_job.remove()
-
-            text = "Alarm silenced."
-        else:
-            # We didn't find an alarm that's currently firing...
-            # Be helpful and check if this is a known reminder without an alarm
-            # currently going off
-            reminder = REMINDERS.get((self.room.room_id, reminder_text.upper()))
-            if reminder:
-                text = (
-                    f"The reminder '{reminder_text}' does not currently have an "
-                    f"alarm going off."
-                )
+            if alarm_job:
+                await self._remove_and_silence_alarm(alarm_job, reminder_text)
+                text = f"Alarm '{reminder_text}' silenced."
             else:
-                text = f"Unknown alarm or reminder '{reminder_text}'."
+                # We didn't find an alarm with that reminder text
+                #
+                # Be helpful and check if this is a known reminder without an alarm
+                # currently going off
+                reminder = REMINDERS.get((self.room.room_id, reminder_text.upper()))
+                if reminder:
+                    text = (
+                        f"The reminder '{reminder_text}' does not currently have an "
+                        f"alarm going off."
+                    )
+                else:
+                    # Nope, can't find it
+                    text = f"Unknown alarm or reminder '{reminder_text}'."
+
+        else:
+            # No reminder text provided. Check if there's a reminder currently firing
+            # in the room instead then
+            for alarm_info, job in ALARMS.items():
+                if alarm_info[0] == self.room.room_id:
+                    # Found one!
+                    reminder_text = alarm_info[
+                        1
+                    ].capitalize()  # normalize the text a bit
+
+                    await self._remove_and_silence_alarm(job, reminder_text)
+                    text = f"Alarm '{reminder_text}' silenced."
+
+                    # Prevent the `else` clause from being triggered
+                    break
+            else:
+                # If we didn't find any alarms...
+                text = "No alarms are currently firing in this room."
 
         await send_text_to_room(self.client, self.room.room_id, text)
 
+    async def _remove_and_silence_alarm(self, alarm_job: Job, reminder_text: str):
+        # We found a reminder with an alarm. Remove it from the dict of current
+        # alarms
+        ALARMS.pop((self.room.room_id, reminder_text.upper()), None)
+
+        if SCHEDULER.get_job(alarm_job.id):
+            # Silence the alarm job
+            alarm_job.remove()
+
+    @command_syntax("")
     async def _list_reminders(self):
         """Format and show known reminders for the current room
 
@@ -383,13 +428,7 @@ class Command(object):
             # Display a recurring time if available
             if reminder.recurse_timedelta:
                 # Get a nice, human-readable version to print
-                human_readable_recurse_timedelta = self._get_readable_timedelta(
-                    reminder.start_time,
-                    pytz.timezone(reminder.timezone),
-                    reminder.recurse_timedelta,
-                )
-
-                line += f" (every {human_readable_recurse_timedelta})"
+                line += f" (every {readabledelta(reminder.recurse_timedelta)})"
 
             # Note that an alarm exists if available
             if reminder.alarm:
@@ -405,9 +444,12 @@ class Command(object):
 
         await send_text_to_room(self.client, self.room.room_id, text)
 
+    @command_syntax("<reminder text>")
     async def _delete_reminder(self):
         """Delete a reminder via its reminder text"""
         reminder_text = " ".join(self.args)
+        if not reminder_text:
+            raise CommandSyntaxError()
 
         logger.debug("Known reminders: %s", REMINDERS)
         logger.debug(
@@ -416,7 +458,7 @@ class Command(object):
 
         reminder = REMINDERS.get((self.room.room_id, reminder_text.upper()))
         if reminder:
-            # Cancel the reminder
+            # Cancel the reminder and associated alarms
             reminder.cancel()
 
             text = "Reminder cancelled."
@@ -425,25 +467,32 @@ class Command(object):
 
         await send_text_to_room(self.client, self.room.room_id, text)
 
+    @command_syntax("")
     async def _help(self):
         """Show the help text"""
+        # Ensure we don't tell the user to use something other than their configured command
+        # prefix
+        c = CONFIG.command_prefix
+
         if not self.args:
             text = (
-                "Hello, I am a reminder bot! Use `help commands` to view available "
-                "commands."
+                f"Hello, I am a reminder bot! Use `{c}help reminders` "
+                f"to view available commands."
             )
             await send_text_to_room(self.client, self.room.room_id, text)
             return
 
         topic = self.args[0]
-        if topic == "commands":
-            text = """
+
+        # Simply way to check for plurals
+        if topic.startswith("reminder"):
+            text = f"""
 **Reminders**
 
 Create an optionally recurring reminder that notifies the reminder creator:
 
 ```
-!remindme [every <recurring time>;] <start time>; <reminder text>
+{c}remindme [every <recurring time>;] <start time>; <reminder text>
 ```
 
 Create an optionally recurring reminder that notifies the whole room.
@@ -451,19 +500,19 @@ Create an optionally recurring reminder that notifies the whole room.
 the room):
 
 ```
-!remindroom [every <recurring time>;] <start time>; <reminder text>
+{c}remindroom [every <recurring time>;] <start time>; <reminder text>
 ```
 
 List all active reminders for a room:
 
 ```
-!listreminders
+{c}listreminders
 ```
 
 Cancel a reminder:
 
 ```
-!cancelreminder <reminder text>
+{c}cancelreminder <reminder text>
 ```
 
 **Alarms**
@@ -472,19 +521,19 @@ Create a reminder that will repeatedly sound every 5m after its usual
 fire time. Otherwise, the syntax is the same as a reminder:
 
 ```
-!alarmme [every <recurring time>;] <start time>; <reminder text>
+{c}alarmme [every <recurring time>;] <start time>; <reminder text>
 ```
 
 or for notifying the whole room:
 
 ```
-!alarmroom [every <recurring time>;] <start time>; <reminder text>
+{c}alarmroom [every <recurring time>;] <start time>; <reminder text>
 ```
 
 Once firing, an alarm can be silenced with:
 
 ```
-!silence <reminder text>
+{c}silence [<reminder text>]
 ```
 
 **Cron-tab Syntax**
@@ -493,13 +542,14 @@ If you need more complicated recurring reminders, you can make use of
 cron-tab syntax:
 
 ```
-!remindme cron <min> <hour> <day of month> <month> <day of week>; <reminder text>
+{c}remindme cron <min> <hour> <day of month> <month> <day of week>; <reminder text>
 ```
 
-This syntax is supported by any `!remind...` or `!alarm...` command above.
+This syntax is supported by any `{c}remind...` or `{c}alarm...` command above.
 """
         else:
-            text = "Unknown help topic!"
+            # Unknown help topic
+            return
 
         await send_text_to_room(self.client, self.room.room_id, text)
 
@@ -508,5 +558,6 @@ This syntax is supported by any `!remind...` or `!alarm...` command above.
         await send_text_to_room(
             self.client,
             self.room.room_id,
-            f"Unknown command '{self.command}'. Try the 'help' command for more information.",
+            f"Unknown help topic '{self.command}'. Try the 'help' command for more "
+            f"information.",
         )
